﻿using System.Runtime.InteropServices;

namespace OBSBlur.Window;

[StructLayout(LayoutKind.Sequential)]
public struct Rectangle
{
   public int Left, Top, Right, Bottom;

   // ReSharper disable once MemberCanBePrivate.Global
   public Rectangle(int left, int top, int right, int bottom)
   {
     Left = left;
     Top = top;
     Right = right;
     Bottom = bottom;
   }

   // ReSharper disable once MemberCanBePrivate.Global
   public Rectangle(System.Drawing.Rectangle r) : this(r.Left, r.Top, r.Right, r.Bottom) { }

   // ReSharper disable once MemberCanBePrivate.Global
   public int X
   {
     get => Left;
     set
     {
       Right -= (Left - value);
       Left = value;
     }
   }

   // ReSharper disable once MemberCanBePrivate.Global
   public int Y
   {
     get => Top;
     set
     {
       Bottom -= (Top - value);
       Top = value;
     }
   }

   // ReSharper disable once MemberCanBePrivate.Global
   public int Height
   {
     get => Bottom - Top;
     set => Bottom = value + Top;
   }

   // ReSharper disable once MemberCanBePrivate.Global
   public int Width
   {
     get => Right - Left;
     set => Right = value + Left;
   }

   public System.Drawing.Point Location
   {
     get => new(Left, Top);
     set
     {
       X = value.X;
       Y = value.Y;
     }
   }

   public System.Drawing.Size Size
   {
     get => new(Width, Height);
     set
     {
       Width = value.Width;
       Height = value.Height;
     }
   }

   public static implicit operator System.Drawing.Rectangle(Rectangle r)
   {
     return new System.Drawing.Rectangle(r.Left, r.Top, r.Width, r.Height);
   }

   public static implicit operator Rectangle(System.Drawing.Rectangle r)
   {
     return new Rectangle(r);
   }

   public static bool operator ==(Rectangle r1, Rectangle r2)
   {
     return r1.Equals(r2);
   }

   public static bool operator !=(Rectangle r1, Rectangle r2)
   {
     return !r1.Equals(r2);
   }

   public bool Equals(Rectangle r)
   {
     return r.Left == Left && r.Top == Top && r.Right == Right && r.Bottom == Bottom;
   }

   public override bool Equals(object? obj)
   {
     if (obj is Rectangle rectangle)
       return Equals(rectangle);
     else if (obj is System.Drawing.Rectangle)
       return Equals(new Rectangle((System.Drawing.Rectangle)obj));
     return false;
   }

   public override int GetHashCode()
   {
     return ((System.Drawing.Rectangle)this).GetHashCode();
   }

   public override string ToString()
   {
     return $"{{L{Left,-6:####0} T{Top,-6:####0} R{Right,-6:####0} B{Bottom,-6:####0}}}";
   }
}