﻿using System.Runtime.InteropServices;
using System.Text;

namespace OBSBlur.Window;

// ReSharper disable once InconsistentNaming
public partial class WindowManager
{
    
    [DllImport("user32.dll", SetLastError = true)]
    static extern IntPtr GetWindow(IntPtr hWnd, uint uCmd);
    
    [DllImport("user32.dll", SetLastError = true)]
    static extern IntPtr GetDesktopWindow();
    
    [DllImport("user32.dll")]
    static extern IntPtr GetTopWindow(IntPtr hWnd);
    
    [DllImport("user32.dll")]
    static extern bool GetWindowRect(IntPtr hWnd, out Rectangle rectangle);
    
    [DllImport("user32.dll")]
    static extern bool IsWindowVisible(IntPtr hWnd);
    
    static List<IntPtr> GetWindowZOrder()
    {
        // ReSharper disable once InconsistentNaming
        const uint GW_HWNDNEXT = 2;
        // ReSharper disable once InconsistentNaming
        const uint GW_HWNDLAST = 1;
        IntPtr desktopWindow = GetDesktopWindow();
        IntPtr topWindow = GetTopWindow(desktopWindow);
        IntPtr bottomWindow = GetWindow(topWindow, GW_HWNDLAST);
        
        IntPtr currentWindow = topWindow;
        List<IntPtr> order = new();
        do
        {
            order.Add(currentWindow);
        } while ((currentWindow = GetWindow(currentWindow, GW_HWNDNEXT)) != bottomWindow);

        return order;
    }
    
    [DllImport("user32.dll")]
    [return: MarshalAs(UnmanagedType.Bool)]
    static extern bool EnumWindows(EnumWindowsProc lpEnumFunc, IntPtr lParam);

    private delegate bool EnumWindowsProc(IntPtr windowHandle, IntPtr lParam);
    
    [DllImport("user32.dll")]
    [return: MarshalAs(UnmanagedType.Bool)]
    // ReSharper disable once MemberCanBePrivate.Global
    public static extern bool GetWindowPlacement(IntPtr windowHandle, ref WindowPlacement lpwndpl);
    
    [DllImport("user32.dll")]
    private static extern uint GetWindowThreadProcessId(IntPtr windowHandle, out uint lpdwProcessId);

    
    [DllImport("user32.dll", CharSet = CharSet.Auto)]
    static extern IntPtr SendMessage(IntPtr windowHandle, uint message, IntPtr wParam, [Out] StringBuilder? lParam);
    
    // ReSharper disable once MemberCanBePrivate.Global
    public static string GetWindowTextRaw(IntPtr windowHandle)
    {
        // ReSharper disable twice InconsistentNaming
        const uint WM_GETTEXTLENGTH = 0x000E;
        const uint WM_GETTEXT = 0x000D;
        // Allocate correct string length first
        int length = (int)SendMessage(windowHandle, WM_GETTEXTLENGTH, IntPtr.Zero, null);
        StringBuilder sb = new (length + 1);
        SendMessage(windowHandle, WM_GETTEXT, sb.Capacity, sb);
        return sb.ToString();
    }
}